/*==============================================================================*/
/*  Copyright (C) 2009-2016, Heartland Data inc. All Rights Reserved.           */
/*                                                                              */
/*  Title  :   Asynchronous Bus Driver                                          */
/*  EventID:   Insert EventID Test Point/Insert Extended EventID Test Point     */
/*  AppVer :   DT10 Ver10.XX`@                                                */
/*  FileID :   11a                                                              */
/*  Version:   1.3                                                              */
/*  Author :   HLDC                                                             */
/*==============================================================================*/

/*==============================================================================*/
/*  Please customize the code for your environment.                             */
/*==============================================================================*/

/*==============================================================================*/
/*  Desc:   Header for FROM Control                                             */
/*==============================================================================*/
#include "include.h"

/*==============================================================================*/
/*  Macro:  DT_UINT                                                             */
/*  Desc:   Please change Test Point argument type for DT10 Project setting.    */
/*==============================================================================*/
#define	DT_UINT unsigned int

/*==============================================================================*/
/*  Macro:  DT_USE_FUNCTION                                                     */
/*  Desc:   Please set 1 when you use function call.                            */
/*==============================================================================*/
#define	DT_USE_FUNCTION 	0

/*==============================================================================*/
/*  Desc:   if you use function call.                                           */
/*          Please set beginning address in the virtual memory space.           */
/*==============================================================================*/
#define	FLASH_ADDR	0xF0000000

/*==============================================================================*/
/*  Macro:  DT_INLINE                                                           */
/*  Desc:   Please use "static" instead of "inline" if "inline" cannot be used. */
/*==============================================================================*/
#define	DT_INLINE	inline
/*#define	DT_INLINE	_inline		/* for Windows	*/
/*#define	DT_INLINE	static		/* when "inline" cannot be used	*/

/*==============================================================================*/
/*  Func:   enterCritical                                                       */
/*  Desc:   Please describe the code to enter Critical Section.                 */
/*==============================================================================*/
DT_INLINE void enterCritical(void)
{
	// ex)	taskENTER_CRITICAL();
	// 
	DI();
}

/*==============================================================================*/
/*  Func:   exitCritical                                                        */
/*  Desc:   Please describe the code to exit Critical Section.                  */
/*==============================================================================*/
DT_INLINE void exitCritical(void)
{
	// ex)	taskEXIT_CRITICAL();
	// 
	EI();
}

/*==============================================================================*/
/*  Func:   _TP_BusOutDrv                                                       */
/*  Desc:   Test Point Output Function                                          */
/*==============================================================================*/
#if (DT_USE_FUNCTION == 1)
DT_INLINE void _TP_BusOutDrv( DT_UINT addr, DT_UINT dat )
{
	addr |= FLASH_ADDR;
	*((volatile short *)addr) = dat;
}
#else
#define	_TP_BusOutDrv( addr, dat )					\
	*((volatile short *)(addr)) = (dat);
#endif

/*==============================================================================*/
/*  Don't change the code from here as possible.                                */
/*==============================================================================*/

/*==============================================================================*/
/*  Desc:   Test Point Parametaer                                               */
/*==============================================================================*/
#define		DT_VARIABLE_BIT			0x02
#define		DT_EVTTRG_BIT			0x08

/*==============================================================================*/
/*  Func:   _TP_BusOut                                                          */
/*  Desc:   Called by Test Point                                                */
/*==============================================================================*/
#if DT_USE_FUNCTION
void _TP_BusOut( DT_UINT addr, DT_UINT dat )
{
	_TP_BusOutDrv( addr, dat );
}
#endif

/*==============================================================================*/
/*  Func:   _TP_MemoryOutput                                                    */
/*  Desc:   Called by Variable TestPoint                                        */
/*==============================================================================*/
void _TP_MemoryOutput( DT_UINT addr, DT_UINT dat, void *value, DT_UINT size )
{
	DT_UINT	i;
	unsigned char *p = value;
	enterCritical();
	_TP_BusOutDrv( addr | DT_VARIABLE_BIT, dat );
	if( size >= 256 ) size = 256;
	for( i = 0; i < size; i += 2 ){
		dat = p[i] ;
		if( (i + 1) < size ){
			dat |= p[i+1] << 8;
		}
#if (DT_USE_FUNCTION == 0)
		addr = FLASH_ADDR | DT_VARIABLE_BIT ;	// Variable Bit
#else
		addr = DT_VARIABLE_BIT ;		// Variable Bit	
#endif
		if( i == 0 ){
			addr |= size << 8 ;			// Size
		}
		_TP_BusOutDrv( addr, dat );
	}
	exitCritical();
}

/*==============================================================================*/
/*  Func:   _TP_EventTrigger                                                    */
/*  Desc:   Called by Event Trigger Test Point(12bit mode)                      */
/*==============================================================================*/
void _TP_EventTrigger( DT_UINT addr, DT_UINT dat, DT_UINT event_id )
{
	enterCritical();
	_TP_BusOutDrv( addr | DT_EVTTRG_BIT, dat );
	_TP_BusOutDrv( addr | DT_EVTTRG_BIT, event_id & 0x00000fff);
	exitCritical();
}

/*==============================================================================*/
/*  Func:   _TP_EventTrigger32                                                  */
/*  Desc:   Called by Event Trigger Test Point(32bit mode)                      */
/*==============================================================================*/
void _TP_EventTrigger32( DT_UINT addr, DT_UINT dat, DT_UINT event_id )
{
	enterCritical();
	_TP_BusOutDrv( addr | DT_EVTTRG_BIT, dat );
	_TP_BusOutDrv( addr | DT_EVTTRG_BIT, (event_id & 0x0000ffff));
	_TP_BusOutDrv( addr | DT_EVTTRG_BIT, ((event_id >> 16) & 0x0000ffff));
	exitCritical();
}

/*==============================================================================*/
